package com.cours.main;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.cours.dao.IAdresseDao;
import com.cours.dao.IPersonneDao;
import com.cours.dao.factory.AbstractDaoFactory;
import com.cours.entities.Adresse;
import com.cours.entities.Personne;

public class MainApp {
	private static final String persistenceUnit = "PersonnesManyToManyPU";
	private static List<Personne> personnes = null;
	private static IPersonneDao personneDao = null;
	private static IAdresseDao adresseDao = null;
	private static List<Adresse> adresses = null;
	private static final Log log = LogFactory.getLog(MainApp.class);

	public static void main(String[] args) {
		//testConfigurationJpaHibernate();
		//displayPersonneAdresses();
		removePersonneAdresse();
		// testSqlDao();
	}

	public static void testConfigurationJpaHibernate() {
		EntityManagerFactory emf = null;
		EntityManager em = null;
		try {
			emf = Persistence.createEntityManagerFactory(persistenceUnit);
			log.debug("CREATION EntityManagerFactory AVEC SUCCES");
			em = emf.createEntityManager();
			log.debug("CREATION EntityManager AVEC SUCCES");
			log.debug("AVANT BEGIN TRANSACTION");
			em.getTransaction().begin();
			log.debug("APRES BEGIN TRANSACTION");
			em.getTransaction().commit();
			log.debug("APRES COMMIT TRANSACTION");
		} catch (Exception e) {
			try {
				// RollBack
				em.getTransaction().rollback();
			} catch (Exception exRollBack) {
				log.error("Une erreur s'est produite lors du RollBack, Exception : " + exRollBack.getMessage()
						+ " , Exception : " + exRollBack);
			}
			log.error("Une erreur s'est produite lors de l'execution la methode, Exception : " + e.getMessage()
					+ " , Exception : " + e);
		} finally {
			// libération ressources
			log.debug("DEBUT FINALEMENT APRES LES CLOSES");
			try {
				if (em != null) {
					em.close();
				}
				if (emf != null) {
					emf.close();
				}
			} catch (Exception e) {
				log.error("Une erreur s'est produite lors de l'execution la methode, Exception : " + e.getMessage()
						+ " , Exception : " + e);
			}
			log.debug("FIN FINALEMENT APRES LES CLOSES");
		}
	}

	public static void displayPersonneAdresses() {
		EntityManagerFactory entityManagerFactory = null;
		EntityManager entityManager = null;
		try {
			entityManagerFactory = Persistence.createEntityManagerFactory(persistenceUnit);
			entityManager = entityManagerFactory.createEntityManager();

			
			Personne person = entityManager.find(Personne.class, 1);
			log.debug("person : "+person);
			List<Adresse> adresses = person.getAdresses();
			log.debug("adresses : "+adresses);
	

			Adresse adresse = entityManager.find(Adresse.class, 1);
			log.debug("adresse : "+adresse);
			List<Personne> personnes = adresse.getPersonnes();
			log.debug("personnes : "+personnes);
			
		} catch (Exception e) {
			log.error("Une erreur s'est produite lors de l'execution la methode, Exception : " + e.getMessage()
					+ " , Exception : " + e);
		} finally {
			if (entityManager != null) {
				entityManager.close();
			}
			if (entityManagerFactory != null) {
				entityManagerFactory.close();
			}
		}
	}

	public static void removePersonneAdresse() {
		EntityManagerFactory entityManagerFactory = null;
		EntityManager entityManager = null;
		EntityTransaction transaction = null;
		try {
			entityManagerFactory = Persistence.createEntityManagerFactory(persistenceUnit);
			entityManager = entityManagerFactory.createEntityManager();

			transaction = entityManager.getTransaction();
			transaction.begin();

			Personne person = entityManager.find(Personne.class, 1);
			log.debug("person : " + person);
			entityManager.remove(person); // Suppression de la personne

			transaction.commit();
		} catch (Exception e) {
			log.error("Une erreur s'est produite lors de l'execution la methode, Exception : " + e.getMessage()
					+ " , Exception : " + e);
		} finally {
			if (entityManager != null) {
				entityManager.close();
			}
			if (entityManagerFactory != null) {
				entityManagerFactory.close();
			}
		}
	}
	
	public static void testSqlDao() {
		// JPA DAO
		personneDao = AbstractDaoFactory.getFactory(AbstractDaoFactory.FactoryDaoType.JPA_DAO).getPersonneDao();
		adresseDao = AbstractDaoFactory.getFactory(AbstractDaoFactory.FactoryDaoType.JPA_DAO).getAdresseDao();
		log.debug("------ personnes : " + personneDao.findAll());

		Personne person = personneDao.findById(5);
		log.debug("personnes (5) : " + person);
		Personne personCrud = new Personne("Nicolas", "Germain", 90.0, 200.0);
		personCrud = personneDao.create(personCrud);
		log.debug("personCrud After create : " + personCrud);
		log.debug("------ personnes After create : " + personneDao.findAll());

		personCrud.setPrenom("Nicolas Bis");
		personCrud.setNom("Germain Bis");
		personCrud = personneDao.update(personCrud);
		log.debug("personCrud After Update : " + personCrud);
		log.debug("------ personnes After Update : " + personneDao.findAll());
		Boolean isOk = personneDao.delete(personCrud);
		personCrud = personneDao.findById(personCrud.getIdPersonne());
		log.debug(" personCrud After Delete isOk : " + isOk + " , personCrud : " + personCrud);
		log.debug("------ personnes After Delete : " + personneDao.findAll());
		// Adresse
		adresses = adresseDao.findAll();
		log.debug("------ adresses : " + adresses);
		Adresse adr = adresseDao.findById(5);
		log.debug("Adresse (5) : " + adr);
		Adresse adresseCrud = new Adresse("rue des heureux", "Nantes", "44000", "France");
		adresseCrud = adresseDao.create(adresseCrud);
		log.debug("adresseCrud After create : " + adresseCrud);
		log.debug("------ personnes After create : " + adresses);

	}
}
